TreeOrderedIndexMapEntry

Struct TreeOrderedIndexMapEntry 

struct TreeOrderedIndexMapEntry {
    element: MutNullableDom<Element>,
    elements: Vec<Dom<Element>>,
    count: usize,
}
An entry in the TreeOrderedIndexMap.

This entry has two states: resolved and unresolved. When an entry in the map refers to a single element, it is resolved. When any additional entries are added or removed, the entry becomes unresolved. At that point, in order to know what element an id or name attribute refer to, the TreeOrderedIndexMap walks the DOM and fills out the elements and element field of the entry.

Fields

element: MutNullableDom<Element>

The first element in the tree that has this key. If this is null, then it means the list needs to be regenerated after a modification of the map.

elements: Vec<Dom<Element>>

An ordered list of Elements that have this key. When this is empty, it means the list needs to be regenerated after a modification of the map.

count: usize

The number of Elements that have this key. This is used in order to do an early return during generation of Self::elements.

Implementations

Source

impl TreeOrderedIndexMapEntry

fn new(element: &Element) -> Self

fn add(&mut self)

fn remove(&mut self) -> bool

fn needs_resolution(&self) -> bool

Returns true if this entry needs to be resolved. An entry will need to be resolved if it ever gains more than one element associated with it or has more than one element associated and Self::remove() is called.